# -*- coding: utf-8 -*-
"""Project metadata

Information describing the project.
"""

# The package name, which is also the "UNIX name" for the project.
package = 'nagios_graphite'
project = "Nagios Graphite"
project_no_spaces = project.replace(' ', '')
version = '0.1.3'
description = 'Nagios command for Graphite metrics'
authors = ['Michael-Keith Bernard']
authors_string = ', '.join(authors)
emails = ['mkbernard.dev@gmail.com']
license = 'MIT'
copyright = '2015 ' + authors_string
url = 'https://github.com/segfaultax/nagios_graphite'
