# -*- coding: utf-8 -*-
"""Nagios command for Graphite metrics"""

from nagios_graphite import metadata


__version__ = metadata.version
__author__ = metadata.authors[0]
__license__ = metadata.license
__copyright__ = metadata.copyright
